// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#pragma once

#include <string>
#include <cstring>
#include <cstdint>
#include <unordered_map>
#include "es_dsp_types.h"
#include "es_nn_common.h"
#include "es_dsp_error.h"
#include "es_dsp_problem.h"

#define KERNEL_MAX_INOUT_TENSOR_NUM 8
#define KERNEL_MAX_C0_BYTES 32

typedef struct es_compiler_buffer_t {
    void* addr;
    uint32_t size;
} es_compiler_buffer;

typedef struct es_compiler_buffer_group_t {
    es_compiler_buffer* buffers;
    uint32_t bufferCnt;
} es_compiler_buffer_group;

typedef struct {
    char operator_name[KERNEL_NAME_MAXLEN];
    char operator_lib_name[KERNEL_LIB_NAME_MAXLEN];

    int32_t input_tensor_num;
    int32_t output_tensor_num;

    ES_DATA_PRECISION_E input_tensor_types[KERNEL_MAX_INOUT_TENSOR_NUM];
    ES_DATA_PRECISION_E output_tensor_types[KERNEL_MAX_INOUT_TENSOR_NUM];

    ES_TENSOR_SHAPE_S input_tensor_shapes[KERNEL_MAX_INOUT_TENSOR_NUM];
    ES_TENSOR_SHAPE_S output_tensor_shapes[KERNEL_MAX_INOUT_TENSOR_NUM];

    // COMMENT: infos to store real struct , infos_kv to store unified key value info.
    es_compiler_buffer_group operator_specific_user_infos;  // eg. softmax need dim to compile the rest params
} dsp_compile_user_info;

typedef enum {
    DSP_COST_PREFER_INFER_TIME,
    DSP_COST_PREFER_POWER_CONSUMPTION,
    DSP_COST_PREFER_INFER_POWER_PRODUCT,
} dsp_cost_mode;

typedef enum {
    DSP_HANDWRITE_OPERATOR,
    DSP_AUTO_GENERTATED_OPERATOR,
} dsp_code_mode;

typedef struct {
    dsp_cost_mode cost_mode;
    dsp_code_mode code_mode;

    dsp_compile_user_info op_user_info;
} dsp_problem_kernel_parsed_desc;

#define ENABLE_HOST_PRINT
#if defined BUILD_RELEASE && BUILD_RELEASE
#define ES_HOST_LOG(format, ...) \
    {                            \
    }
#else
#define ES_HOST_LOG(format, ...)                   \
    do {                                           \
        if (format) printf(format, ##__VA_ARGS__); \
    } while (0)
#endif

#define ES_HOST_ERROR(format, ...)                 \
    do {                                           \
        if (format) printf(format, ##__VA_ARGS__); \
    } while (0)

/*
    int32_t cost;  // ret for predicted dsp operator infer time or power consumption
    // ....
    // every operator implement a compile_dsp_desc for specific dsp_compile_info .
    //       while compile_dsp_desc is invoked , this func fills desc and params
    //
    return cost;
*/
#ifdef __cplusplus
extern "C" {
#endif

extern int32_t parse_yml2desc(std::unordered_map<std::string, std::string> yml_map, ES_DSP_OP_PROBLEM_S* problem);
extern int32_t emit_dsp_operator_desc(dsp_problem_kernel_parsed_desc* problem, ES_DSP_OPERATOR_DESC_S* op_desc);

#ifdef __cplusplus
}
#endif