// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#pragma once
#include <dirent.h>
#include <dlfcn.h>
#include <memory>
#include <string>
#include <unordered_map>
#include <vector>
#include <functional>
#include <iostream>
#include "compile_registry.h"

namespace op
{
namespace compile
{

class Manager
{
 private:
    Manager();
    std::unordered_map<std::string, std::vector<std::unique_ptr<Operator>>> Opmap;
    Manager(Manager const&) = delete;
    void operator=(Manager const&) = delete;

 public:
    std::unordered_map<std::string, std::vector<std::unique_ptr<Operator>>>& registeredOp();
    void printList();
    static Manager& getInstance();

    ParseFunc getParseParamFunc(const std::string& opName);
};

class OpRegistry
{
 public:
    explicit OpRegistry(const char* name);

    Operator& getOperator();

 private:
    Operator& opRef;
};

class LoadOpLibs
{
 public:
    explicit LoadOpLibs(const char* folder_path);

    ~LoadOpLibs();

 private:
    std::vector<void*> handle_;
};

}  // namespace compile
}  // namespace op

#define OP_STR_CONCAT_(__x, __y, __z) __x##__y##__z
#define OP_STR_CONCAT(__x, __y, __z) OP_STR_CONCAT_(__x, __y, __z)

#define OP_REGISTER_GLOBAL(name)                                                  \
    static auto& OP_STR_CONCAT(__op_registry_instance__, __COUNTER__, __##name) = \
        ::op::compile::OpRegistry(#name).getOperator()

#define LOAD_OP_LIBS(op_path) static ::op::compile::LoadOpLibs LOAD_OP_LIBS_instance(op_path)