// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#pragma once
#include <dlfcn.h>
#include <memory>
#include <utility>
#include <string>
#include <unordered_map>
#include <vector>
#include <functional>
#include <iostream>
#include "es_dsp_problem.h"

namespace op
{
namespace compile
{
using Problem = ES_DSP_OP_PROBLEM_S;
using OperatorDesc = ES_DSP_OPERATOR_DESC_S;
using PrecisionMap = std::vector<std::pair<ES_DATA_PRECISION_E, ES_DATA_PRECISION_E>>;
using ParseFunc = std::function<int32_t(const std::unordered_map<std::string, std::string> &, Problem *)>;
using ConsFunc = std::function<bool(Problem problem, PrecisionMap map)>;
using EmitFunc = std::function<ES_S32(Problem problem, OperatorDesc **opDesc)>;
using CostFunc = std::function<ES_DSP_COST_DESC_S(Problem &problem)>;

class Operator
{
 public:
    Operator() = default;

    bool satisfyConstraints(Problem problem);

    ES_DSP_COST_DESC_S evalCost(Problem &problem);

    int32_t emitOperatorDesc(Problem problem, OperatorDesc **opDesc);

    Operator &setDefaultFuncs(ConsFunc cons, CostFunc cost, EmitFunc emit, ParseFunc parse);

    Operator &setConstrainFunc(ConsFunc cons);

    Operator &setCostFunc(CostFunc cost);

    Operator &setEmitFunc(EmitFunc emit);

    Operator &setParseFunc(ParseFunc parse);

    Operator &setPrecisionMap(PrecisionMap map);

    ParseFunc &getParseFunc();

    static Operator &opRegister(std::string name);
    class Manager;

 private:
    std::string opName;

    ConsFunc satisfyConstraintsFunc;
    CostFunc evalCostFunc;
    EmitFunc emitOperatorDescFunc;
    ParseFunc parseParamFunc;
    PrecisionMap precisionMap;
    friend class Manager;
};

}  // namespace compile
}  // namespace op
