// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef __ESWIN_DSP_PERFORMANCE_API_H__
#define __ESWIN_DSP_PERFORMANCE_API_H__

#include "es_dsp_ll_api.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif

/**
 * @brief Enable performance statistics switch.
 *
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LL_EnablePerformance(ES_S32 dspFd);

/**
 * @brief Get performance data.
 *
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LL_GetPerformanceData(ES_S32 dspFd, void *data);

/**
 * @brief Get dsp hardware performance data.
 *
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LL_GetHwPerformanceData(ES_S32 dspFd, void *data);

/**
 * @brief Disable performance statistics switch.
 *
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LL_DisablePerformance(ES_S32 dspFd);

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif

#endif